// DamePanel.java                               MM 2009
// MVC-View

import java.awt.*;                   // Color, Graphics
import javax.swing.*;                         // JPanel

/**
 * Schachbrettdarzustellung einer Dameloesung.
 */

public class DamePanel
  extends JPanel
{
/* ------------------------------------------------- */
                                            // coreData
/**
 * Darzustellende Loesung.
 */
  private DameLoesung loesung;

/* ------------------------------------------------- */
                     // Konstanten fuer die Darstellung
/**
 * Brettbreite in Pixel.
 */
  private final int brettBreite = 400;           // 400

/**
 * Rand oben in Pixel.
 */
  private final int top = 20;                    //  20

/**
 * Rand Feldbreite in Pixel.
 */
  private final int feldBreite                   //  40
  = (int)((brettBreite - 2 * top) / 9);

/**
 * Steindurchmesser in Pixel.
 */
  private final int steinBreite = feldBreite -2; //  36

/**
 * Rand links in Pixel.
 */
  private final int left = top + feldBreite;     //  60

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Stellt Schachbrett mit Loesung dar.
 * @param loesung darzustellende Loesung.
 */
  public DamePanel( DameLoesung loesung)
  {
    super();
    this.loesung = loesung;
    setPreferredSize
    ( new Dimension( brettBreite, brettBreite));
  }

/**
 * Stellt Schachbrett ohne Loesung dar.
 */
  public DamePanel()
  {
    this( null);
  }

/* ------------------------------------------------- */
                                    // service-Methoden
/**
 * Setzt Loesung.
 * @param loesung darzustellende Loesung.
 */
  public void setLoesung( DameLoesung loesung)
  {
    this.loesung = loesung;
    repaint();
  }

/**
 * Zeichnet Schachbrett ohne/mit Loesung.
 */
  public void paint( Graphics g)
  {
    int i, j;

// Leeren des Darstellungsbereich
    g.setColor( getBackground());
    g.fillRect( 0, 0, getWidth(), getHeight());

// Brett ohne Loesung
    g.setColor( Color.black);
    g.fillRect
    ( left, top, 8 * feldBreite, 8 * feldBreite);

    for( i = 0; i < 8; i++)
      for( j = i%2; j < 8; j += 2)
      {
        g.setColor( Color.white);
        g.fillRect( left + j * feldBreite,
                    top + i * feldBreite,
                    feldBreite, feldBreite);
      }

// Beschriftung 'A' .. 'H'
    for( i = 0; i < 8; i++)
    {
      g.setColor( Color.black);
      g.drawString( "" + (char)('A' + i),
                    (i + 2) * feldBreite,
                    brettBreite - 3 / 2 * top);
    }

// Beschriftung '1' .. '8'
    for( j = 0; j < 8; j++)
    {
      g.setColor( Color.black);
      g.drawString( "" + (j + 1),
                   left / 2,
                   brettBreite - (j + 2) * feldBreite);
    }

// Loesung
    for( i = 0; i < 8; i++)
    {
      if( loesung != null)
      { g.setColor( Color.red);
        j = (loesung.getSpalten())[ i];
        g.fillOval( left + 1 + i * feldBreite,
                    brettBreite -
                    ( top - 1 + (j + 2) * feldBreite),
                    steinBreite, steinBreite);
      }
    }
  }
}