// DameView.java                                MM 2009
// MVC-View

import javax.swing.*;                // GUI-Komponenten
import java.util.*;                         // Observer

/**
 * View zum Damenproblem.
 */

public class DameView
  extends JFrame
  implements Observer
{
/* ------------------------------------------------- */
                                      // GUI-Konstanten
                                                 // GUI
/**
 * Konstante, Berechnung wiederholen.
 */
  public final String ACTION_NEU = "Neu";

/**
 * Konstante, Berechnung starten.
 */
  public final String ACTION_START = "Start ";

/**
 * Konstante, Loesung anzeigen.
 */
  public final String ACTION_WEITER = "Weiter";

/**
 * Konstante, alle Loesungen angezeigt.
 */
  public final String ACTION_FERTIG = "Fertig";

/**
 * Konstante, Programm beenden.
 */
  public final String ACTION_ENDE = "Beenden";

/* ------------------------------------------------- */
                                     // GUI-Komponenten
/**
 * Button Start, Weiter, Fertig.
 * mit wechselnder Aufschrift.
 */
  private JButton btStart;

/**
 * Button Neu.
 */
  private JButton btNeu;

/**
 * Button Beenden.
 */
  private JButton btEnde;

/**
 * Schachbrett zur Ausgabe der Loesungen.
 */
  private DamePanel schachBrett;

/**
 * Label "Loesungsnummer: ".
 */
  private JLabel lbNr;

/**
 * Loesungsnummer.
 */
  private int nr;

/* ------------------------------------------------- */
                                                 // MVC
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet des Damenproblems.
 */
  private DameModel model;

/**
 * Zum View gehoeriger Cotroller.
 */
  private DameController controller;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift
 */
  public DameView( DameModel model, String titel)
  {
// Titelzeile
    setTitle( titel);

// Model
    this.model = model;
    this.model.addObserver( this);      // Ueberwachung

// Controller
    controller = makeController();

// View
    makeView();
  }

/**
 * Erzeugt Controller, Empfaenger fuer Ereignisse.
 * @return Controller fuer View
 */
  private DameController makeController()
  {
    return new DameController( model, this);
  }

/**
 * Erzeugt View,
 * baut die grafische Oberflaeche auf,
 * Controller verarbeitet Fensterereignisse.
 */
  private void makeView()
  {
// ContentPane
    getContentPane().add( createDameBox());

// Startzustand
    reset();

// Fenster
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                         // ContentPane
/**
 * Senkrechte Box fuer Schachbrett und Button.
 */
  private Box createDameBox()
  {
    schachBrett = new DamePanel();

    Box vbox = Box.createVerticalBox();
    vbox.add( Box.createVerticalStrut( 10));
    vbox.add( schachBrett);
    vbox.add( Box.createVerticalStrut( 10));
    vbox.add( createLoesungsNummer());
    vbox.add( Box.createVerticalStrut( 10));
    vbox.add( createButtons());
    vbox.add( Box.createVerticalStrut( 10));

    return vbox;
  }

/**
 * Box fuer Label mit Lsungsnummer.
 */
  private Box createLoesungsNummer()
  {
    JLabel label = new JLabel( "Lsungsnummer: ");
    lbNr = new JLabel( "" + nr);

    Box box = Box.createHorizontalBox();
    box.add( label);
    box.add( lbNr);

    return box;
  }

/**
 * Waagerechte Box fuer Steuerbutton,
 * Controller verarbeitet Buttonereignisse.
 */
  private Box createButtons()
  {
    btStart = new JButton( ACTION_START);
    btStart.addActionListener( controller);

    btNeu = new JButton( ACTION_NEU);
    btNeu.addActionListener( controller);

    btEnde = new JButton( ACTION_ENDE);
    btEnde.addActionListener( controller);

    Box box = Box.createHorizontalBox();
    box.add( btStart);
    box.add( btNeu);
    box.add( btEnde);

    return box;
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                    // service-Methoden fuer Controller
/**
 * Startzustand, Button Neu.
 */
  public void reset()
  {
// Loesungsnummer
    nr = 0;
    lbNr.setText( "" + nr);

// Button
    btStart.setText( ACTION_START);
    btStart.setEnabled( true);
    btStart.requestFocus();

// Schachbrett
    schachBrett.setLoesung( null);
  }

/**
 * Start, startet Anzeige der Loesungen.
 */
  private void setBtStart()
  {
    btStart.setText( ACTION_WEITER);
    btStart.requestFocus();
    setBtWeiter();
  }

/**
 * Weiter, stellt Loesung zur Anzeige bereit.
 */
  public void setBtWeiter()
  {
    Menge<DameLoesung> loesungen = model.getLoesungen();
    DameLoesung loesung;

    if( nr == 0) loesung = loesungen.erstesElement();
    else loesung = loesungen.naechstesElement();

    if( loesung != null)
    {
      nr++;
      lbNr.setText( "" + nr);

      schachBrett.setLoesung( loesung);
    }
    else setBtFertig();
  }

/**
 * Fertig, alle Loesungen sind angezeigt.
 */
  private void setBtFertig()
  {
    btStart.setText( ACTION_FERTIG);
    btStart.setEnabled( false);
    btNeu.requestFocus();
  }

/* ------------------------------------------------- */
                                    // Observer-Methode
/**
 * Ueberschreiben der Interfacemethode update,
 * alle Loesungen wurden vom Modell gefunden.
 * @param m Model, welches Aenderungen meldet
 * @param o geaenderte Objekte
 */
  public void update( Observable m, Object o)
  {
    if( model == m) setBtStart();
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Entfernt Fenster, deinstalliert MVC.
 */
  public void release()
  {
// Fenster
    dispose();

// Controller
    controller.release();
    controller = null;

// Model
    model.deleteObserver( this);
    model = null;
  }
}
